#ifndef _MELEE_H
#define _MELEE_H

#include <stdio.h>
#include <allegro.h>
#include <math.h>
#ifndef PI
  #define PI 3.1415926535
#endif

#ifdef _WIN32
//  #include <winalleg.h>
  #define random()  rand()
  #define srandom(a) srand(a)
#endif


struct JGMOD;

#define MELEE_ASTEROID          0
#define MELEE_ASTEROIDEXPLOSION 64
#define MELEE_BOOM              84
#define MELEE_BOOMSHIP          88
#define MELEE_HOTSPOT           89
#define MELEE_KABOOM            101
#define MELEE_MUSIC             121
#define MELEE_PANEL             122
#define MELEE_PLANET            125
#define MELEE_SPARK             128
#define MELEE_STAR              134

#define GLOBAL_MAXSPEED 1.0

#define SCPPAL_PALLETE 0
#define SCPPAL_FONT    4

#define X_MAX 3840.0
#define Y_MAX 3840.0

#define MAX_SHIPS      8
#define MAX_STARS      100
#define MAX_ASTEROIDS  8
#define MAX_HOTSPOTS   20

#define SPACE_LAYERS 7

#define LAYER_HOTSPOTS   0
#define LAYER_CBODIES    1
#define LAYER_LINES      2
#define LAYER_SHOTS      3
#define LAYER_SHIPS      4
#define LAYER_SPECIAL    5
#define LAYER_EXPLOSIONS 6


#define MAX_SPACE_ITEMS (2 * (1 + MAX_ASTEROIDS + (MAX_SHIPS * (32 + MAX_HOTSPOTS)) + (MAX_STARS * 3)))
#define MAX_RADAR_ITEMS (2 * (1 + MAX_ASTEROIDS + MAX_SHIPS))

#define MAX_FLEET_SIZE 50
#define EMPTY_SLOT     -1

extern FILE *debug_file;

extern char *key_string[];
int key_num (const char *name);

extern FONT *desktop_font;

#define CAPTAIN_WIDTH   55
#define CAPTAIN_HEIGHT  30

#define RADAR_COLOR_FRIEND   2
#define RADAR_COLOR_ENEMY    4
#define RADAR_COLOR_PLANET   9
#define RADAR_COLOR_ASTEROID 8

#define bit(a) (1 << (a))

#define BOOM_SAMPLES 4

#define ANGLE_RATIO     (3.1415926535 / 180.0)//radians per degree
#define RANGE_RATIO     40.0					//???
extern double distance_ratio;
extern int time_ratio;
int scale_frames(double value);
double scale_range(double range);
double scale_turning (double turn_rate) ;
double scale_velocity (double velocity) ;
double scale_acceleration (double acceleration, double hotspot_rate) ;


#define ASTEROID_FRAMES          64
#define ASTEROIDEXPLOSION_FRAMES 20
#define HOTSPOT_FRAMES           12
#define KABOOM_FRAMES            20
#define SPARK_FRAMES             6

#define HOT_COLORS 12

#define PHASE_MAX   12
#define PHASE_DELAY 50

extern int hot_color[HOT_COLORS];

//melee/mframe.h
class SpaceFrame;
class SpaceSprite;
class SpaceLocation;
class SpaceObject;
class SpaceLine;

//melee/mcontrol.h
class Control;
class ControlWrapper;

//melee/mship.h, melee/mshpdata, melee/mshppan.h
class Ship;
class ShipData;
class ShipPanel;

//melee/mview.h
class View;

//frame.h
struct DirtyItem;
class Frame;

//melee/mmain.h
extern DATAFILE *melee;
extern int debug_value;
extern SpaceFrame *spaceFrame;

//ppcol.h
struct PPCOL_MASK;

//melee/mshot.
class Shot;
class AnimatedShot;
class Missile;
class HomingMissile;
class Laser;

//melee/mcbodies.
class Animation;
class Asteroid;
class Planet;

//
extern SpaceSprite *asteroidSprite;
extern SpaceSprite *asteroidExplosionSprite;
extern SpaceSprite *hotspotSprite;
extern SpaceSprite *kaboomSprite;
extern SpaceSprite *sparkSprite;

extern JGMOD *meleeVictory;

#define MAX_SHIP_TYPES 100


class SpaceSprite {
	protected:
	int         count;
	int         w;
	int         h;
	PPCOL_MASK **m;
	BITMAP     **b;

	public:
	SpaceSprite(DATAFILE *sprites, int sprite_count);
	SpaceSprite(BITMAP **bmp, int bmp_count);

	virtual ~SpaceSprite();

	BITMAP     *get_bitmap(int index);

	virtual void animate(int x, int y, int index, double zoom, Frame *space);

	void draw_character(int x, int y, int index, int color, BITMAP *bmp);
	void draw_character_stretch(int x, int y, int index, int color, double zoom, BITMAP *bmp);
	void animate_character(int x, int y, int index, int color, double zoom, Frame *space);

	int collide(int x, int y, int i, int ox, int oy, int oi,
			SpaceSprite *other);
	int collide_line(int lx1, int ly1, int *lx2, int *ly2, int sx, int sy,
			int sindex);

	int    frames();
	double width();
	double height();
	};

class SpaceRotatingSprite : public SpaceSprite {
  public:
  SpaceRotatingSprite(DATAFILE *sprites, int sprite_count);
};

typedef struct {
  char *id;
  char *name;
  Ship *(*getShip)(double sX, double sY, double sAngle, int sCollideFlag,
    int sRadarColor, ShipData *sData, Control *cntrl);
} SHIP_REGISTER_INFO;

typedef struct {
  SHIP_REGISTER_INFO *reg;
  ShipData           *dat;
  int                cost;
} SHIP_INFO;

void register_ship(SHIP_REGISTER_INFO *reg_info);
void init_ships();

void tw_error(char* message);

char *shipListboxGetter(int index, int *list_size);
char *fleetListboxGetter(int index, int *list_size);

double normalize(double value, double max);
double wcorner(double x, double w);
double hcorner(double y, double h);
double min_delta(double from, double to, double max);
double nearest_coord(double from, double to, double max);
double distance_from(double x, double y, double to_x, double to_y);


int tw_sample (SAMPLE *spl, int vol = 256, int pan = 128, int freq = 1000) ;

class ShipData {
  public:
  DATAFILE    *data;
  BITMAP     **bitmapPanel;
  SpaceSprite *spriteShip;
  SpaceSprite *spriteWeapon;
  SpaceSprite *spriteWeaponExplosion;
  SpaceSprite *spriteSpecial;
  SpaceSprite *spriteSpecialExplosion;
  SpaceSprite *spriteExtra;
  SpaceSprite *spriteExtraExplosion;
  SAMPLE     **sampleWeapon;
  SAMPLE     **sampleSpecial;
  SAMPLE     **sampleExtra;
  JGMOD       *moduleVictory;

  ShipData(char *filename);
  ~ShipData();
};
#endif
