#ifndef _MFRAME_H
#include "mframe.h"
#endif

#ifndef _MSHIP_H
#define _MSHIP_H

extern double gravity_sling;

struct keyflag {
	enum { 
		left = 1, 
		right = 2, 
		thrust = 4, 
		fire = 8, 
		special = 16, 
		next = 32, 
		prev = 64, 
		close = 128
		};
	};

class Ship : public SpaceObject {
  protected:
  SpaceSprite *phaseSprite;

  int hotspot_frame;
  int phase_count;
  int phase_step;
  int recharge_step;
  int weapon_recharge;
  int weapon_low;
  int special_recharge;
  int special_low;

  virtual void materialize();

  virtual void calculate_phase();
  virtual void calculate_thrust();
  virtual void calculate_turn_left();
  virtual void calculate_turn_right();
  virtual void calculate_fire_weapon();
  virtual void calculate_fire_special();
  virtual void calculate_hotspots();

  virtual int activate_weapon();
  virtual int activate_special();

  double target_x;
  double target_y;

  public:
  int    crew;
  int    crew_max;
  int    batt;
  int    batt_max;
  double turn_rate;
  double turn_step;
  double speed_max;
  double accel_rate;
  int    recharge_amount;
  int    recharge_rate;
  int    weapon_drain;
  int    weapon_rate;
  int    weapon_sample;
  int    special_drain;
  int    special_rate;
  int    special_sample;
  int    hotspot_rate;
	double gravity_vx;
	double gravity_vy;

  BITMAP   *panelBitmap[7];
  ShipData *data;
  Ship     *target;

  int update_panel;

  int nextkeys;

  int thrust;
  int turn_left;
  int turn_right;
  int fire_weapon;
  int fire_special;
	int target_next;
	int target_prev;
	int target_close;

	int target_pressed;
	Control *control;

  friend class ShipPanel;

  Ship(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);
	virtual void death();

  virtual int getCrew();
  virtual int getBatt();
  virtual int crewPanelColor();
  virtual int battPanelColor();

  void locate();
  
	virtual int accelerate (SpaceLocation *source, double angle, double vel, double max_speed);

  void calculate();
  void animate(Frame *space);
  void animate_radar(Frame *radar);
  void handle_damage(SpaceLocation *source);
};

#endif