
#include "../melee.h"
#include "../id.h"
#include "mmain.h"
#include "mshot.h"
#include "mship.h"
#include "manim.h"

// Define shot spark
SpaceSprite *sparkSprite;

Shot::Shot(double ox, double oy, double oangle, double ov, int odamage,
  double orange, int oarmour, Ship *oship, SpaceLocation *opos,
  SpaceSprite *osprite, double relativity) :
  SpaceObject(opos->normal_x(), opos->normal_y(), oangle,
    oship->allyFlag(), osprite),
  v(ov),
  d(0.0),
  range(orange),
  armour(oarmour),
  explosionSprite(sparkSprite),
  explosionSample(NULL),
  explosionFrameCount(SPARK_FRAMES),
  explosionFrameSize(scale_frames(0)),
  ship(oship)
{
  layer = LAYER_SHOTS;
	double alpha, tx, ty;
	alpha = (opos->get_angle() * ANGLE_RATIO);
	tx = cos(alpha);
	ty = sin(alpha);
// angle conventions fucked up??
	x += oy * tx - ox * ty;
	y += oy * ty + ox * tx;

  vx = (v * cos(angle * ANGLE_RATIO)) + opos->get_vx() * relativity;
  vy = (v * sin(angle * ANGLE_RATIO)) + opos->get_vy() * relativity;

  damage_factor = odamage;

  id = SPACE_SHOT;

	if (range < 0) range = 999999999.0;
}

void Shot::calculate() {
	SpaceObject::calculate();
	d += v * frame_time;
	if(d >= range) state = 0;
	if (ship && !ship->exists()) {
//		ship = NULL;
		state = 0;
		}
	return;
	}

void Shot::handle_damage(SpaceLocation *source) {
	if((damage > 0) || (directDamage > 0)) {
		armour -= damage;
		armour -= directDamage;
		damage = 0;
		directDamage = 0;
		if(armour <= 0) {
			armour = 0;
			state = 0;
			animateExplosion();
			soundExplosion();
			}
		}
	}

void Shot::inflict_damage(SpaceObject *other) {
	SpaceObject::inflict_damage(other);
	if (other->mass > 0) state = 0;
	if (state == 0) animateExplosion();
	}

void Shot::animateExplosion() {
	spaceFrame->addSpaceObject(new Animation(normal_x(),
			normal_y(), explosionSprite, explosionFrameCount,
			explosionFrameSize, LAYER_EXPLOSIONS));
	}

void Shot::soundExplosion() {
	if(explosionSample) {
		stop_sample(explosionSample);
		tw_sample(explosionSample);
		}
	}

void Shot::stop() {
	vx = 0.0;
	vy = 0.0;
	}

void Shot::destroy() {
	state = 0;
	}

AnimatedShot::AnimatedShot(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship, SpaceLocation *opos,
  SpaceSprite *osprite, int ofcount, int ofsize, double relativity) :
  Shot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, opos, osprite, relativity),
  frame_count(ofcount),
  frame_size(ofsize),
  frame_step(ofsize)
{
}

void AnimatedShot::calculate() {
	Shot::calculate();
	frame_step -= frame_time;
	while (frame_step < 0) {
		frame_step += frame_size;
		sprite_index++;
		if (sprite_index == frame_count) sprite_index = 0;
		}
	return;
	}

Missile::Missile(double ox, double oy, double oangle, double ov, int odamage,
  double orange, int oarmour, Ship *oship, SpaceLocation *opos,
  SpaceSprite *osprite, double relativity) :
  Shot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, opos, osprite, relativity)
{
  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;
}

HomingMissile::HomingMissile(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, double otrate, Ship *oship,
  SpaceSprite *osprite) :
  Missile(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite, 0),
  turn_rate(otrate),
  turn_step(0.0),
	target(oship->target)
{
}

void HomingMissile::calculate()
{
//  double alpha = angle;

  Missile::calculate();

  if (target && !target->isAlly(ship)) {
//    alpha = trajectory_angle(target);
//    if(((alpha - angle) <= 180.0) && ((alpha - angle) >= 0.0) ||
//       ((alpha - angle) <= (-180.0)))
		double dx = min_delta(target->normal_x(), normal_x(), X_MAX);
	  double dy = min_delta(target->normal_y(), normal_y(), Y_MAX);
		if (dy * vx - dx * vy > 0)
      turn_step += turn_rate * frame_time;
    else
      turn_step -= turn_rate * frame_time;
  }

  while(fabs(turn_step) > 5.625) {
    if(turn_step < 0.0) {
      angle -= 5.625;
      turn_step += 5.625;
    } else
    if(turn_step > 0.0) {
      angle += 5.625;
      turn_step -= 5.625;
    }

    if(angle < 0.0)
      angle += 360.0;
    if(angle >= 360.0)
      angle -= 360.0;
  }

  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

  vx = v * cos(angle * ANGLE_RATIO);
  vy = v * sin(angle * ANGLE_RATIO);
}

Laser::Laser(double langle, int lcolor, double lrange, int ldamage,
  int lfcount, SpaceLocation *opos, double rel_x, double rel_y) :
  SpaceLine(0.0, 0.0, langle, opos->allyFlag(), lrange, lcolor),
  frame(0),
  frame_count(lfcount),
  pos(opos),
  pos_x(rel_x),
  pos_y(rel_y)
{
  double alpha;

  id            = SPACE_LASER;
  damage_factor = ldamage;

	alpha = (pos->get_angle() * ANGLE_RATIO);
	double tx, ty;
	tx = cos(alpha);
	ty = sin(alpha);
// angle conventions fucked up??
	x = pos->normal_x() + pos_y * tx - pos_x * ty;
	y = pos->normal_y() + pos_y * ty + pos_x * tx;

	if(!pos->exists()) state = 0;
}


void Laser::calculate() {
	double alpha, tx, ty;
	if((frame < frame_count) && (pos->exists())) {
		alpha = (pos->get_angle() * ANGLE_RATIO);
		tx = cos(alpha);
		ty = sin(alpha);
// angle conventions fucked up??
		x = pos->normal_x() + pos_y * tx - pos_x * ty;
		y = pos->normal_y() + pos_y * ty + pos_x * tx;
		SpaceLine::calculate();
		frame += frame_time;
		}
	else state = 0;
	return;
	}
