#ifndef _MFRAME_H
#include "mframe.h"
#endif

#ifndef _MSHOT_H
#define _MSHOT_H


class Shot : public SpaceObject {
  protected:
  double v;
  double d;
  double range;
  int    armour;
  
  SpaceSprite *explosionSprite;
  SAMPLE      *explosionSample;
  int          explosionFrameCount;
  int          explosionFrameSize;

  public:
  Ship  *ship;

  Shot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceLocation *opos,
    SpaceSprite *osprite, double relativity = 0.5);

  virtual void calculate();
  virtual void handle_damage(SpaceLocation *source);
  virtual void inflict_damage(SpaceObject *other);

  virtual void animateExplosion();
  virtual void soundExplosion();

  void stop();
  void destroy();
};

class AnimatedShot : public Shot {
  protected:
  int frame_count;
  int frame_size;
  int frame_step;

  public:
  AnimatedShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceLocation *opos,
    SpaceSprite *osprite, int ofcount, int ofsize, double relativity = 0.5);

  virtual void calculate();
};

class Missile : public Shot {
  public:
  Missile(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceLocation *opos,
    SpaceSprite *osprite, double relativity = 0.5);
};

class HomingMissile : public Missile {
  protected:
  double turn_rate;
  double turn_step;

  public:
  HomingMissile(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);

  virtual void calculate();
	SpaceLocation *target;
};

class Laser : public SpaceLine {
  protected:
  double frame;
  double frame_count;

  SpaceLocation *pos;
  double pos_x;
  double pos_y;

  public:
  Laser(double langle, int lcolor, double lrange, int ldamage, int lfcount,
    SpaceLocation *opos, double rel_x, double rel_y);

  void calculate();
};


#endif