
#include <allegro.h>

#include "../melee.h"
#include "mmain.h"
#include "mshppan.h"
#include "mship.h"

int PANELS_PER_COLUMN = 4;
int PANEL_WIDTH = 64;
int PANEL_HEIGHT = 100;

void overlay_bitmap(BITMAP *src1, BITMAP *src2, BITMAP *dest, int w, int h);

ShipPanel::ShipPanel(Ship *ship, int shipIndex)
{
  panel   = create_bitmap(64, 100);
  captain = create_bitmap(CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

  initialize(ship, shipIndex);
}

ShipPanel::~ShipPanel()
{
  #ifdef SCP_DEBUG
	fprintf(debug_file, "? ShipPanel -> destructor\n");
	fflush(debug_file);
	#endif
	acquire_screen();
  rectfill(screen, captain_x, captain_y,
    captain_x + CAPTAIN_WIDTH, captain_y + CAPTAIN_HEIGHT, 0);
	release_screen();

  destroy_bitmap(captain);
  destroy_bitmap(panel);
  #ifdef SCP_DEBUG
	fprintf(debug_file, "? ShipPanel -> destructor done\n");
	fflush(debug_file);
	#endif
}

int ShipPanel::exists()
{

  if((ship == NULL) || (fryFrame > 0))
		return 0;
	return 1;
}

void ShipPanel::refresh(int index)
{
	#ifdef SCP_DEBUG
	fprintf(debug_file, "? ShipPanel -> refresh\n");
	fflush(debug_file);
	#endif
  initialize(ship, index);
	#ifdef SCP_DEBUG
	fprintf(debug_file, "? ShipPanel -> refresh done\n");
	fflush(debug_file);
	#endif
}

void ShipPanel::initialize(Ship *oship, int shipIndex)
{
  crew_old   = 0;
  batt_old   = 0;
  batt_light = FALSE;

  ship     = oship;
  fryFrame = 0;

 	#ifdef SCP_DEBUG
  fprintf(debug_file, "? ShipPanel -> initialize\n");
  fflush(debug_file);
	#endif
  old_thrust       = FALSE;
  old_turn_left    = FALSE;
  old_turn_right   = FALSE;
  old_fire_weapon  = FALSE;
  old_fire_special = FALSE;

  panel_x = SCREEN_W - 64 * (1 + shipIndex / PANELS_PER_COLUMN);
  panel_y = 100 * (shipIndex % PANELS_PER_COLUMN);
  blit(ship->panelBitmap[0], panel, 0, 0, 0, 0, 64, 100);
  acquire_screen();
  blit(panel, screen, 0, 0, panel_x, panel_y, 64, 100);
	release_screen();

  captain_x = panel_x + 4;
  captain_y = panel_y + 65;
  captain_animated = FALSE;
}

void ShipPanel::calculate()
{
  if(fryFrame > 0) {
    fryFrame--;
    #ifdef SCP_DEBUG
		fprintf(debug_file, "x ShipPanel -> calculate, fryFrame decrement: %d", fryFrame);
		fflush(debug_file);
    #endif
		} 
	else if(ship != NULL) {
		if(!ship->exists()) {
			acquire_screen();
			rectfill(screen, panel_x + 4, panel_y + 54 - ship->crew_max,
					panel_x + 9, panel_y + 54, 0);
			rectfill(screen, panel_x + 52, panel_y + 54 - ship->batt_max,
					panel_x + 57, panel_y + 54, 0);
			release_screen();
			ship = NULL;
	  
			#ifdef SCP_DEBUG
			fprintf(debug_file, "? ShipPanel -> FryFrame initialized\n");
			fflush(debug_file);
			#endif
      
			if(fryFrame == 0) fryFrame = 100; 
			} 
		else {
			if(ship->update_panel) {
				ship->update_panel = FALSE;
        refresh(panel_y / 100);
				#ifdef SCP_DEBUG
				fprintf(debug_file, "? ShipPanel -> refresh return\n");
				fflush(debug_file);
				#endif
        return;
				}
      if(
        (ship->thrust != old_thrust) ||
        (ship->turn_left != old_turn_left) ||
        (ship->turn_right != old_turn_right) ||
        (ship->fire_weapon != old_fire_weapon) ||
        (ship->fire_special != old_fire_special))
					captain_animated = FALSE;
    }
  }
}

void ShipPanel::animate()
{
  int crew_x = 8;
  int crew_y = 53;
  int batt_x = 56;
  int batt_y = 53;

  int crew_color = 0;
  int batt_color = 0;

  int i, bar_x, bar_y;
  double w, h;

  if(fryFrame > 0) {
		#ifdef SCP_DEBUG
		fprintf(debug_file, "x ShipPanel -> Fryframe animate: %d\n", fryFrame);
    fflush(debug_file);
    #endif
    rectfill(captain, 0, 0, CAPTAIN_WIDTH, CAPTAIN_HEIGHT, 0);
    w = (double)(CAPTAIN_WIDTH * (double)(fryFrame)) / 100.0;
    h = (double)(CAPTAIN_HEIGHT * (double)(fryFrame)) / 100.0;
    if(w < 1.0) w = 1.0;
    if(h < 1.0) h = 1.0;
    rectfill(captain,
				(int)(((double)(CAPTAIN_WIDTH) - w) / 2.0),
				(int)(((double)(CAPTAIN_HEIGHT) - h) / 2.0),
				(int)((double)(CAPTAIN_WIDTH) - (((double)(CAPTAIN_WIDTH) - w) / 2.0)),
				(int)((double)(CAPTAIN_HEIGHT) - (((double)(CAPTAIN_HEIGHT) - h) / 2.0)),
				pallete_color[128 - (fryFrame / 16)]);
		acquire_screen();
		blit(captain, screen, 0, 0, captain_x, captain_y,
				CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
		release_screen();
		captain_animated = FALSE;
		#ifdef SCP_DEBUG
		fprintf(debug_file, "x ShipPanel -> Fryframe animate done\n");
		fflush(debug_file);
		#endif
		}
	else
	if(ship != NULL) {
		crew_color = ship->crewPanelColor();
    batt_color = ship->battPanelColor();

    if(ship->crew != crew_old) {
      bar_x = 0;
      bar_y = 0;
      for(i = 0; i < ship->crew_max; i++) {
        if((i - ship->crew) < 0) {
          putpixel(panel, crew_x + bar_x, crew_y + bar_y, crew_color);
          putpixel(panel, crew_x + bar_x + 1, crew_y + bar_y, crew_color);
        } else {
          putpixel(panel, crew_x + bar_x, crew_y + bar_y, 0);
          putpixel(panel, crew_x + bar_x + 1, crew_y + bar_y, 0);
        }
        if((i % 2) == 0)
          bar_x = -3;
        else {
          bar_x = 0;
          bar_y -= 2;
        }
      }
      acquire_screen();
      blit(panel, screen, crew_x - 3, crew_y - ship->crew_max + 1,
        panel_x + crew_x - 3, panel_y + crew_y - ship->crew_max + 1, 5,
        ship->crew_max + 1);
			release_screen();
      crew_old = ship->crew;
    }

    if(ship->batt != batt_old) {
      bar_x = 0;
      bar_y = 0;
      for(i = 0; i < ship->batt_max; i++) {
        if((i - ship->batt) < 0) {
          putpixel(panel, batt_x + bar_x, batt_y + bar_y, batt_color);
          putpixel(panel, batt_x + bar_x + 1, batt_y + bar_y, batt_color);
        } else {
          putpixel(panel, batt_x + bar_x, batt_y + bar_y, 0);
          putpixel(panel, batt_x + bar_x + 1, batt_y + bar_y, 0);
        }
        if((i % 2) == 0)
          bar_x = -3;
        else {
          bar_x = 0;
          bar_y -= 2;
        }
      }
      acquire_screen();
      blit(panel, screen, batt_x - 3, batt_y - ship->batt_max + 1,
        panel_x + batt_x - 3, panel_y + batt_y - ship->batt_max + 1, 5,
        ship->batt_max + 1);
			release_screen();
      batt_old = ship->batt;
    }

    if((!batt_light) && (ship->weapon_low || ship->special_low)) {
      acquire_screen();
      draw_rle_sprite(screen, (RLE_SPRITE *)(melee[MELEE_PANEL + 2].dat),
        panel_x + 40, panel_y + 58);
			release_screen();
      batt_light = TRUE;
    } else
    if(batt_light && (!ship->weapon_low) && (!ship->special_low)) {
      acquire_screen();
			draw_rle_sprite(screen, (RLE_SPRITE *)(melee[MELEE_PANEL + 1].dat),
        panel_x + 40, panel_y + 58);
			release_screen();
      batt_light = FALSE;
    }

    if(!captain_animated) {
      blit(ship->panelBitmap[1], captain, 0, 0, 0, 0,
        CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

      if(ship->thrust)
        overlay_bitmap(ship->panelBitmap[1], ship->panelBitmap[2], captain,
          CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

      if((ship->turn_left) && (!ship->turn_right))
        overlay_bitmap(ship->panelBitmap[1], ship->panelBitmap[3], captain,
          CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

      if((ship->turn_right) && (!ship->turn_left))
        overlay_bitmap(ship->panelBitmap[1], ship->panelBitmap[4], captain,
          CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

      if(ship->fire_weapon)
        overlay_bitmap(ship->panelBitmap[1], ship->panelBitmap[5], captain,
          CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

      if(ship->fire_special)
        overlay_bitmap(ship->panelBitmap[1], ship->panelBitmap[6], captain,
          CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

      acquire_screen();
      blit(captain, screen, 0, 0, captain_x, captain_y,
        CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
			release_screen();
      captain_animated = TRUE;
    }

    old_thrust       = ship->thrust;
    old_turn_left    = ship->turn_left;
    old_turn_right   = ship->turn_right;
    old_fire_weapon  = ship->fire_weapon;
    old_fire_special = ship->fire_special;
  } else
  if(!captain_animated) {
		acquire_screen();
    rectfill(screen, captain_x, captain_y, captain_x + CAPTAIN_WIDTH,
      captain_y + CAPTAIN_HEIGHT, 0);
		release_screen();
    captain_animated = TRUE;
  }
}

void overlay_bitmap(BITMAP *src1, BITMAP *src2, BITMAP *dest, int w, int h)
{
  int x, y;

  for(y = 0; y < h; y++) {
    for(x = 0; x < w; x++) {
      if(getpixel(src1, x, y) != getpixel(src2, x, y))
        putpixel(dest, x, y, getpixel(src2, x, y));
    }
  }
}
