
#include "../shipstuf.h"

# define NORMAL_SPEED 0
# define TURBO_SPEED 1
# define SUPER_SPEED 2
# define HYPER_SPEED 3

# define SPECIAL_UP 0
# define TURRENT_DOWN 1
# define TURBO_DOWN 2

class KterbiSaber : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       specialRange;
  double       specialVelocity;
  int          specialDamage;
  int          specialArmour;
  int          extraspeed;
  int          boostlevel;
  int          normal_recharge;
  double       normal_speed;
  int          normal_rate;
  int          speedcount;
  int          turbo;
  int          specialHeldDown;
  int          specialcount;

  public:
  KterbiSaber(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void calculate_fire_weapon();
  virtual void calculate_fire_special();
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();

};

class KterbiIonBlast : public AnimatedShot {

  int min_damage;

  public:
  KterbiIonBlast(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  virtual void calculate();
};

KterbiSaber::KterbiSaber(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  specialRange   = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity= scale_velocity(get_config_float("Special", "Velocity", 0));
  specialDamage  = get_config_int("Special", "Damage", 0);
  specialArmour  = get_config_int("Special", "Armour", 0);
  extraspeed     = get_config_int("Extra","Speed",0);
  boostlevel     = NORMAL_SPEED;
  speedcount     = 0;
  normal_recharge= recharge_amount;
  normal_rate    = recharge_rate;
  normal_speed   = speed_max;
  turbo          = FALSE;
  specialHeldDown= FALSE;
  specialcount   = 0;

  collide_flag_ally |= bit(LAYER_SHOTS);
}

void KterbiSaber::calculate_fire_weapon()
{
  if ((batt < weapon_drain) && (crew > 1)) {
    crew--;
    batt += batt_max; }
  Ship::calculate_fire_weapon();
  if ((batt > batt_max) && (crew < crew_max)) {
    crew++;
    batt -= batt_max;
  }
  if (batt > batt_max)
    batt = batt_max;
}

void KterbiSaber::calculate_fire_special()
{
  if ((batt < special_drain) && (crew > 1)) {
    crew--;
    batt += batt_max; }
  Ship::calculate_fire_special();
  if ((batt > batt_max) && (crew < crew_max)) {
    crew++;
    batt -= batt_max;
  }
  if (batt > batt_max)
    batt = batt_max;
}

int KterbiSaber::activate_weapon()
{
  if (!fire_special) {
  spaceFrame->addSpaceObject(new KterbiIonBlast(-width()*.35, height()/3,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon, 10 ,1));
  spaceFrame->addSpaceObject(new KterbiIonBlast(width()*.35, height()/3,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon, 10 ,1));
  return(TRUE); }
  else return(FALSE);
}

int KterbiSaber::activate_special()
{
  int special_used;
  int fire = FALSE, i;
  SpaceObject *o;
  SpaceObject *closest;

  if ((turbo) && (specialHeldDown!=TURRENT_DOWN)){
    specialHeldDown = TURBO_DOWN;
    special_used = FALSE;
    speedcount = 0;
    boostlevel++;
    stop_sample(data->sampleExtra[1]);
    tw_sample(data->sampleExtra[1]);
    turbo = FALSE;
    speedcount = 0;
      if (boostlevel == TURBO_SPEED) {
        speed_max = normal_speed * 2;
        recharge_step = 0;
        recharge_amount = 0;
      } else if (boostlevel == SUPER_SPEED) {
        recharge_amount = -normal_recharge;
        recharge_step = 0;
        speed_max = normal_speed * 3;
      } else if (boostlevel == HYPER_SPEED) {
        recharge_rate = (normal_rate/2);
        speed_max = normal_speed * 4;
        recharge_step= 0;
    } }
    else if (specialHeldDown != TURBO_DOWN) {
      specialHeldDown = TURRENT_DOWN;
      if (fire_weapon) {
        specialcount ++;
        if (specialcount > 1) {
          specialcount = 0;
          batt += special_drain;
        }
        special_used=TRUE;
        closest = NULL;
        for(i = 0; i < SPACE_LAYERS; i++) {
          o = spaceFrame->getFirstObject(i);
          while(o != NULL) {
            if( (i != LAYER_HOTSPOTS) && ((o->getID() & SPACE_MASK) != SPACE_PLANET)
              && ((o->getID() & SPACE_MASK) != SPACE_SHIP) && (i != LAYER_EXPLOSIONS)
              && (!o->isAlly(this)) && (distance(o) > 1) && (distance(o) < specialRange)) {
              fire = TRUE;
	      if ((closest == NULL) || (distance(o) < distance(closest)))
                  closest=o;
            }
            o = o->nexto;
          }
        }
      } else return FALSE;
      if (fire) {
        stop_sample(data->sampleExtra[2]);
        tw_sample(data->sampleExtra[2]);
      	double a;
        double r = distance(closest);
        r = r / specialVelocity;
	double x = min_delta( normal_x() , closest->normal_x(), X_MAX);
	double y = min_delta( normal_y() , closest->normal_y(), Y_MAX);
	a = atan2(y, x) * 180 / PI - 180;
        spaceFrame->addSpaceObject(new Shot(0.0, 0.0,
            a , specialVelocity, specialDamage, specialRange,
            specialArmour, this, this, data->spriteSpecial));
      } } else {special_used = FALSE; }
    return(special_used);
}



void KterbiSaber::calculate()
{
  if (((!thrust) || ((crew == 1) && (batt == 0))) && (boostlevel != 0)) {
      speedcount = 0;
      boostlevel = 0;
      recharge_amount = normal_recharge;
      recharge_rate = normal_rate;
      speed_max = normal_speed;
      recharge_step = 0;
      turbo = FALSE;
      }

  if (!fire_special) {
    specialcount = 0;
    specialHeldDown = FALSE;
  }

  if ((!thrust) && (turbo)) {
    speedcount = 0;
    turbo = FALSE;
  }

  Ship::calculate();

  if ((batt < 0) && (crew!=1)) {
    batt += batt_max;
    crew--;
  } else if (batt < 0)
    batt=0;

  if ((crew!=crew_max) && (batt >= batt_max)) {
    batt -= batt_max;
    crew++;
  }
  if ((thrust) && (!turbo)) {
    speedcount++;
    if ((speedcount > extraspeed) && (boostlevel != 3)) {
      stop_sample(data->sampleExtra[0]);
      tw_sample(data->sampleExtra[0]);
      turbo = TRUE;
    }
  }
}

KterbiIonBlast::KterbiIonBlast(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship,
      osprite, ofcount, ofsize)
{
  min_damage=odamage;
}

void KterbiIonBlast::calculate()
{
  AnimatedShot::calculate();
  if (d < (range/2))
    damage_factor = min_damage;
  else
    damage_factor = min_damage * 2;
}

Ship *getKterbiSaber(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new KterbiSaber(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO KterbiSaberInfo =
{
  "ktesa",
  "Kterbi Saber",
  getKterbiSaber
};
