
#include "../shipstuf.h"

#define MAX_POISONS 8

class NaroolLurker : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
	int          normalRechargeAmount;
  int cloak;
  int cloak_frame;

  public:

  Ship         *poisons[MAX_POISONS];

  static int cloak_color[3];

  NaroolLurker(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int isAlly(SpaceLocation *other);
  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void calculate_hotspots();
  virtual void calculate();
  virtual void animate(Frame *space);
  virtual void animate_radar(Frame *radar);
};
class NaroolGas: public AnimatedShot {

  int hitShip;
  NaroolLurker *creator;

  public:

  NaroolGas(double ox,double oy,double oangle,double ov, int odamage,
     double orange,int oarmour, NaroolLurker *oship, SpaceSprite *osprite);

  virtual void inflict_damage(SpaceObject *other);
  virtual void soundExplosion();
  virtual void animateExplosion();
};

class NaroolPoison : public SpaceObject {
  Ship *oship;
  NaroolLurker *creator;
  SpaceSprite *sprite;
  int start;

  public:

  NaroolPoison(NaroolLurker *creator, Ship *oship, SpaceSprite *osprite);
  virtual void calculate();
  virtual void animate(Frame *space);
  virtual void animate_radar(Frame *radar);
};

int NaroolLurker::cloak_color[3] = { 15, 10, 2 };

NaroolLurker::NaroolLurker(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

	normalRechargeAmount = recharge_amount;

  cloak = FALSE;
  cloak_frame = 0;
  int i;
  for (i=0; i<MAX_POISONS; i++)
    poisons[i]=NULL;
}

int NaroolLurker::isAlly(SpaceLocation *other)
{
  if(cloak_frame >= 300) return(TRUE);
  return (Ship::isAlly(other));
}

int NaroolLurker::activate_weapon()
{
	double a;
	if (target) {
		double r = distance(target);
		r = r / weaponVelocity;
		double x = min_delta( normal_x() , target->normal_x() - 
				(get_vx()/2 - target->get_vx()) * r, X_MAX);
		double y = min_delta( normal_y() , target->normal_y() - 
				(get_vy()/2 - target->get_vy()) * r, Y_MAX);
		a = atan2(y, x) * 180 / PI - 180;
		}
	else a = angle;
  spaceFrame->addSpaceObject(new NaroolGas(0.0, 0.0,
    a , weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon));
  return(TRUE);
}

void NaroolLurker::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if((batt < special_drain) && (!cloak)) {
      special_low = TRUE;
      return;
    }

    if(special_recharge > 0)
      return;

    if(cloak) {
      cloak = FALSE;

      stop_sample(data->sampleSpecial[1]);
      tw_sample(data->sampleSpecial[1]);
			recharge_amount = normalRechargeAmount;
    } else {
      cloak = TRUE;
      stop_sample(data->sampleSpecial[0]);
      tw_sample(data->sampleSpecial[0]);
      batt -= special_drain;
			recharge_amount = 1;
    }

    special_recharge = special_rate;
  }
}

void NaroolLurker::calculate_hotspots()
{
  if(!cloak)
    Ship::calculate_hotspots();
}

void NaroolLurker::calculate()
{
  if((cloak) && (cloak_frame < 300))
    cloak_frame+= frame_time;
  if((!cloak) && (cloak_frame > 0))
    cloak_frame-= frame_time;

  Ship::calculate();
}

void NaroolLurker::animate(Frame *space) {
	if(phase_count < PHASE_MAX)
		return;

	if((cloak_frame > 0) && (cloak_frame < 300))
		sprite->animate_character((int)(corner_x()), (int)(corner_y()),
				sprite_index, pallete_color[cloak_color[(int)(cloak_frame / 100)]], space_zoom, space);
	else
	if((cloak_frame >= 300))
		sprite->animate_character((int)(corner_x()), (int)(corner_y()),
				sprite_index, pallete_color[0], space_zoom, space);
	else
		Ship::animate(space);
	}

void NaroolLurker::animate_radar(Frame *radar) {
	if(phase_count < PHASE_MAX)
		return;

	if(cloak_frame <= 0) Ship::animate_radar(radar);
	}

NaroolGas::NaroolGas(double ox, double oy,double oangle,double ov,int odamage,
  double orange,int oarmour, NaroolLurker *oship, SpaceSprite *osprite) :
  AnimatedShot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite, 50, 21)

{  
	creator = oship;
	hitShip = FALSE;
	collide_flag_anyone = bit(LAYER_SHIPS) | bit(LAYER_CBODIES);
}

void NaroolGas::inflict_damage(SpaceObject *other)
{
  int i=0,inlist=FALSE;
  Ship *force;
  if (other->isShip())
    {
      force= (Ship *) other;
      while ((i < MAX_POISONS) && (!inlist) && (creator->poisons[i] != NULL))
        if (!(inlist=(creator->poisons[i] == force)))
          i++;
      if ((i < MAX_POISONS) && !inlist)
        {
        spaceFrame->addSpaceObject(new NaroolPoison(creator,force,creator->data->spriteWeapon));
        creator->poisons[i]=force;
        }
    }
  hitShip = TRUE;
  AnimatedShot::inflict_damage(other);
  spaceFrame->addSpaceObject(new FixedAnimation(other,
    creator->data->spriteWeaponExplosion, 8, 150, LAYER_EXPLOSIONS));
  stop_sample(creator->data->sampleExtra[1]);
  tw_sample(creator->data->sampleExtra[1]);

}

void NaroolGas::animateExplosion()
{
  if (!hitShip) {
  spaceFrame->addSpaceObject(new Animation(x, y,
    creator->data->spriteWeaponExplosion, 8, 150, LAYER_EXPLOSIONS));
  }
}

void NaroolGas::soundExplosion()
{
	if (!hitShip) {
		stop_sample(creator->data->sampleExtra[1]);
		tw_sample(creator->data->sampleExtra[1]);
	}
}

NaroolPoison::NaroolPoison(NaroolLurker *ncreator,Ship *nship, SpaceSprite *osprite):
  SpaceObject(nship->normal_x(),nship->normal_y(),0.0,ncreator->allyFlag(),
    osprite),
  oship(nship),
  creator(ncreator)
  {
	start = TRUE;
	collide_flag_anyone = 0;
	}

void NaroolPoison::calculate() {
  int i, slide = FALSE ,chance;

  if (!creator->exists())
    {
      blit(oship->data->bitmapPanel[0], oship->panelBitmap[0],
        16 , 18, 16, 18, 32, 30);
      oship->update_panel = TRUE;
      creator = NULL;
      state = 0;
			return;
    }
	if (!oship->exists()) {
		for (i=0; i<MAX_POISONS-1; i++) {
			if (slide) creator->poisons[(i-1)] = creator->poisons [i];
			if (creator->poisons[i] == oship) slide = TRUE;
			}
		if (slide) creator->poisons[MAX_POISONS-1] = NULL;
		oship = NULL;
		state = 0;
		}
	else {
		if (start) {
			start = FALSE;
			draw_sprite(oship->panelBitmap[0], creator->data->spriteExtra->get_bitmap(0),16,18);
			oship->update_panel = TRUE;
			}
		chance = random() % 2000;
		if (chance < frame_time) {
			oship->directDamage++;
			tw_sample(creator->data->sampleExtra[0]);
			oship->handle_damage(creator);
			}
		}
	return;
	}

void NaroolPoison::animate(Frame *space)
{ 
}

void NaroolPoison::animate_radar(Frame *radar)
{ 
}

Ship *getNaroolLurker(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new NaroolLurker(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO NaroolLurkerInfo =
{
  "narlu",
  "Narool Lurker",
  getNaroolLurker
};
