
#include "../shipstuf.h"

class VirtaoLimb : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurnRate;

  double specialRange;
  double specialPower;

  public:
  VirtaoLimb(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class VirtaoMissile : public HomingMissile {
  public:
  VirtaoMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

VirtaoLimb::VirtaoLimb(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponTurnRate = scale_turning(get_config_float("Weapon", "TurnRate", 0));

  specialRange  = scale_range(get_config_float("Special", "Range", 0));
  specialPower  = scale_range(get_config_float("Special", "Power", 0));

}

int VirtaoLimb::activate_weapon() {
  spaceFrame->addSpaceObject(new VirtaoMissile(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, weaponTurnRate, this, data->spriteWeapon));
  return(TRUE);
}

int VirtaoLimb::activate_special() {
  int fire = FALSE, i;
  SpaceObject *o;

  for(i = 0; i < SPACE_LAYERS; i++) {
    o = spaceFrame->getFirstObject(i);
    while(o != NULL) {
			if( (!o->isAlly(this)) && (distance(o) < specialRange) &&
					canCollide(o) && o->canCollide(this)) {
				double a = trajectory_angle(o) * PI / 180;
				o->translate(cos(a) * 300, sin(a) * specialPower);
				fire = TRUE;
				}
			o = o->nexto;
			}
		}
	if (fire) tw_sample((SAMPLE *)(melee[MELEE_BOOM + 0].dat));

	return(fire);
	}

VirtaoMissile::VirtaoMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate, 
  Ship *oship, SpaceSprite *osprite) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship, osprite)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 10;
  explosionFrameSize  = 50;
}

Ship *getVirtaoLimb(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new VirtaoLimb(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO VirtaoLimbInfo =
{
  "virli",
  "Virtao Host",
  getVirtaoLimb
};
