
#include "../shipstuf.h"

class AndrosynthGuardian : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  SpaceSprite *specialSprite;
  SpaceSprite *shipSprite;
  double       specialVelocity;
  double       specialTurnRate;
  int          specialDamage;
  int          specialActive;
  double       shipTurnRate;
  int          shipRechargeAmount;

  public:
  AndrosynthGuardian(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void calculate();
  virtual void calculate_thrust();
  virtual void calculate_hotspots();

  virtual void inflict_damage(SpaceObject *other);

  virtual int activate_weapon();
  virtual int activate_special();
};

class AndrosynthBubble : public AnimatedShot {
  int courseFrames;

  public:
  AndrosynthBubble(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  void calculate();
};

AndrosynthGuardian::AndrosynthGuardian(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponDamage       = get_config_int("Weapon", "Damage", 1);
  weaponArmour       = get_config_int("Weapon", "Armour", 1);
  weaponRange        = scale_range(get_config_float("Weapon", "Range", 10));
  weaponVelocity     = scale_velocity(get_config_float("Weapon", "Velocity", 100));

  specialSprite   = data->spriteSpecial;
  shipSprite      = sprite;
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialTurnRate = scale_turning(get_config_float("Special", "TurnRate", 0));
  specialActive   = FALSE;

  shipTurnRate    = turn_rate;
	shipRechargeAmount = recharge_amount;
}

void AndrosynthGuardian::calculate()
{
	Ship::calculate();
	if(specialActive && (batt == -1)) {
		sprite = shipSprite;
		w      = shipSprite->width();
		h      = shipSprite->height();
		damage_factor = 0;
		specialActive = FALSE;
		vx = 0.0;
		vy = 0.0;
		turn_rate = shipTurnRate;
		recharge_amount = shipRechargeAmount;
		batt = 0;
		}
	return;
	}

void AndrosynthGuardian::calculate_thrust()
{
  if(specialActive) {
    vx = specialVelocity * cos(angle * ANGLE_RATIO);
    vy = specialVelocity * sin(angle * ANGLE_RATIO);
  } else
    Ship::calculate_thrust();
}

void AndrosynthGuardian::calculate_hotspots()
{
  if(specialActive)
    return;
  Ship::calculate_hotspots();
}

void AndrosynthGuardian::inflict_damage(SpaceObject *other)
{
  if(damage_factor > 0)
    spaceFrame->addSpaceObject(new Animation(
      x + (cos(trajectory_angle(other) * ANGLE_RATIO) * 20.0),
      y + (sin(trajectory_angle(other) * ANGLE_RATIO) * 20.0),
      sparkSprite, SPARK_FRAMES, 50, LAYER_EXPLOSIONS));
  SpaceObject::inflict_damage(other);
}

int AndrosynthGuardian::activate_weapon() {
  
  if(specialActive)
	  return FALSE;

  spaceFrame->addSpaceObject(new AndrosynthBubble(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon, 10, 50));
  return(TRUE);
}

int AndrosynthGuardian::activate_special() {
  if(specialActive)
    return(FALSE);

  if (batt < 1) return(FALSE);

  sprite = specialSprite;
  w      = specialSprite->width();
  h      = specialSprite->height();

  damage_factor = specialDamage;
  specialActive = TRUE;
  turn_rate     = specialTurnRate;
  
  recharge_step = recharge_rate;
	recharge_amount = -1;

  return(TRUE);
}

AndrosynthBubble::AndrosynthBubble(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite, ofcount, ofsize),
  courseFrames(0)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 6;
  explosionFrameSize  = 50;
}

void AndrosynthBubble::calculate()
{
  double newAngle;

  AnimatedShot::calculate();
  courseFrames += frame_time;
  if(courseFrames >= 150) {
    courseFrames -= 150;
    
    newAngle = (double)(random() % 360);
    vx = v * cos(newAngle * ANGLE_RATIO);
    vy = v * sin(newAngle * ANGLE_RATIO);

    int i;
		double r = 99999;
		newAngle = 99999;
		if (ship) for (i = 0; i < ship->control->num_targets; i += 1) {
			Ship *s = ship->control->target[i];
			if (s && (distance(s) < r)) {
				r = distance(s);
				newAngle = trajectory_angle(s);
				}
			}
    if(newAngle < 720) {
      vx += v * cos(newAngle * ANGLE_RATIO) / 2;
      vy += v * sin(newAngle * ANGLE_RATIO) / 2;
      }      
		}
	return;
	}

Ship *getAndrosynthGuardian(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new AndrosynthGuardian(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}


SHIP_REGISTER_INFO AndrosynthGuardianInfo =
{
  "andgu",
  "Androsynth Guardian",
  getAndrosynthGuardian
};

