
#include "../shipstuf.h"

class EarthlingCruiser : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurnRate;

  int    specialColor;
  double specialRange;
  int    specialFrames;
  int    specialDamage;

  public:
  EarthlingCruiser(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class EarthlingMissile : public HomingMissile {
  public:
  EarthlingMissile(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, Ship *oship,
    SpaceSprite *osprite);
};

EarthlingCruiser::EarthlingCruiser(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl) {

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponTurnRate = scale_turning(get_config_float("Weapon", "TurnRate", 0));

  specialColor  = get_config_int("Special", "Color", 0);
  specialRange  = scale_range(get_config_float("Special", "Range", 0));
  specialFrames = get_config_int("Special", "Frames", 0);
  specialDamage = get_config_int("Special", "Damage", 0);

	collide_flag_ally = bit(LAYER_SHOTS);
}

int EarthlingCruiser::activate_weapon() {
  spaceFrame->addSpaceObject(new EarthlingMissile(
    0.0, (height() * 1.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, weaponTurnRate, this, data->spriteWeapon));
  return(TRUE);
}

int EarthlingCruiser::activate_special() {
  int fire = FALSE;
  SpaceObject *o;

	Query a;
	for (a.begin(this, bit(LAYER_SHIPS) + bit(LAYER_SHOTS) + bit(LAYER_SPECIAL) + 
			bit(LAYER_CBODIES), weaponRange + 200); a.current; a.next()) {
		o = a.currento;
		if( (!o->isAlly(this)) && (distance(o) < specialRange) &&
			canCollide(o) && o->exists()) {
			SpaceLine *l = spaceFrame->addSpaceLine(new Laser(trajectory_angle(o),
					pallete_color[specialColor], distance(o), 0,
					specialFrames, this, 0.0, 10.0));
			if (o->canCollide(l)) {
				o->damage += specialDamage;
				o->handle_damage(this);
				fire = TRUE;
				}
			else l->state = 0;
			}
		}
	if (fire) tw_sample((SAMPLE *)(melee[MELEE_BOOM + 0].dat));

	return(fire);
	}

EarthlingMissile::EarthlingMissile(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, double otrate,
  Ship *oship, SpaceSprite *osprite) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, oarmour, otrate, oship,
    osprite)
{
	collide_flag_ally = bit(LAYER_SHIPS) | bit(LAYER_SHOTS);
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 10;
  explosionFrameSize  = 50;
}

Ship *getEarthlingCruiser(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new EarthlingCruiser(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO EarthlingCruiserInfo =
{
  "earcr",
  "Earthling Cruiser",
  getEarthlingCruiser
};
