
#include "../shipstuf.h"

class IlwrathAvenger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int cloak;
  int cloak_frame;

  public:
	static int cloak_color[3];
  IlwrathAvenger(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int isAlly(SpaceLocation *other);
  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void calculate_hotspots();
  virtual void calculate();
  virtual void animate(Frame *space);
  virtual void animate_radar(Frame *radar);
};
int IlwrathAvenger::cloak_color[3] = { 15, 11, 9 };

IlwrathAvenger::IlwrathAvenger(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  cloak = FALSE;
  cloak_frame = 0;
}

int IlwrathAvenger::isAlly(SpaceLocation *other)
{
  if(cloak_frame >= 300)
    return(TRUE);
  return(SpaceLocation::isAlly(other));
}

int IlwrathAvenger::activate_weapon()
{
  if (cloak && (target != NULL)) angle = trajectory_angle(target);
  cloak = FALSE;
  spaceFrame->addSpaceObject(new AnimatedShot(0.0, (h / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    this, data->spriteWeapon, 50, 12));
  return(TRUE);
}

void IlwrathAvenger::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if((batt < special_drain) && (!cloak)) {
      special_low = TRUE;
      return;
    }

    if(special_recharge != 0)
      return;

    if(cloak) {
      cloak = FALSE;

      stop_sample(data->sampleSpecial[1]);
      tw_sample(data->sampleSpecial[1]);
    } else {
      cloak = TRUE;
      stop_sample(data->sampleSpecial[0]);
      tw_sample(data->sampleSpecial[0]);
      batt -= special_drain;
    }

    special_recharge = special_rate;
  }
}

void IlwrathAvenger::calculate_hotspots()
{
  if(!cloak)
    Ship::calculate_hotspots();
}

void IlwrathAvenger::calculate()
{
  if((cloak) && (cloak_frame < 300))
    cloak_frame += frame_time;
  if((!cloak) && (cloak_frame > 0))
    cloak_frame -= frame_time;

  Ship::calculate();
}

void IlwrathAvenger::animate(Frame *space)
{
  if(phase_count < PHASE_MAX)
    return;

  if((cloak_frame > 0) && (cloak_frame < 300))
    sprite->animate_character((int)(corner_x()), (int)(corner_y()),
      sprite_index, pallete_color[cloak_color[cloak_frame / 100]], space_zoom, space);
  else
  if((cloak_frame >= 300))
    sprite->animate_character((int)(corner_x()), (int)(corner_y()),
      sprite_index, pallete_color[0], space_zoom, space);
  else {
		Ship::animate(space);		
		}
}
void IlwrathAvenger::animate_radar(Frame *space) {
  if(phase_count < PHASE_MAX)
    return;
  if(cloak_frame <= 0) Ship::animate_radar(radar);
	}


Ship *getIlwrathAvenger(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new IlwrathAvenger(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO IlwrathAvengerInfo =
{
  "ilwav",
  "Ilwrath Avenger",
  getIlwrathAvenger
};
