
#include "../shipstuf.h"

class KzerZaDreadnought : public Ship {
  double weaponRange;
  double weaponVelocity;
  int    weaponDamage;
  int    weaponArmour;

  int    specialFrames;
  int    specialLaserDamage;
  int    specialLaserColor;
  double specialLaserRange;
  int    specialLaserFrames;
  int    specialLaserDrain;
  double specialVelocity;
  double specialRange;
  int    specialArmour;

  public:
  KzerZaDreadnought(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  int activate_weapon();
  int activate_special();
};

class KzerZaMissile : public Missile {
  public:
  KzerZaMissile(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite);
};

class KzerZaFighter : public Missile {
  int    air_frames;
  int    max_air_frames;
  int    laser_damage;
  int    laser_color;
  double laser_range;
  int    laser_frames;
  int    batt;
  int    recharge_frames;

  public:
  KzerZaFighter(double ox, double oy, double oangle, double ov,
    double orange, int oarmour, int oair_frames,
    int olaser_damage, int olaser_color, double olaser_range,
    int olaser_frames, int odrain, Ship *oship, SpaceSprite *osprite);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage(SpaceLocation *source);
};

KzerZaDreadnought::KzerZaDreadnought(double ox, double oy, double shipAngle,
  int shipRadarColor, int shipCollideFlag, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipRadarColor, shipCollideFlag, shipData, cntrl) {

	collide_flag_ally = 1 << LAYER_SHOTS;

  weaponRange         = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity      = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage        = get_config_int("Weapon", "Damage", 0);
  weaponArmour        = get_config_int("Weapon", "Armour", 0);

  specialFrames          = get_config_int("Special", "Frames", 0);
  specialLaserDamage     = get_config_int("Special", "LaserDamage", 0);
  specialLaserColor      = get_config_int("Special", "LaserColor", 0);
  specialLaserRange      = scale_range(get_config_float("Special", "LaserRange", 0));
  specialLaserFrames     = get_config_int("Special", "LaserFrames", 0);
  specialLaserDrain      = get_config_int("Special", "LaserDrain", 0);
  specialVelocity        = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialRange           = scale_range(get_config_float("Special", "Range", 0));
  specialArmour          = get_config_int("Special", "Armour", 0);
}

int KzerZaDreadnought::activate_weapon()
{
  spaceFrame->addSpaceObject( new KzerZaMissile(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon));
  return(TRUE);
}

int KzerZaDreadnought::activate_special()
{
  if(crew > 1) {
    spaceFrame->addSpaceObject( new KzerZaFighter(-25.0, -50.0,
      angle - 157.5, specialVelocity, specialRange, specialArmour,
      specialFrames, specialLaserDamage,
      specialLaserColor, specialLaserRange, specialLaserFrames,
      specialLaserDrain, this, data->spriteSpecial));
    crew--;
    if(crew > 1) {
      spaceFrame->addSpaceObject( new KzerZaFighter(25.0, -50.0,
        angle + 157.5, specialVelocity, specialRange, specialArmour,
        specialFrames, specialLaserDamage,
        specialLaserColor, specialLaserRange, specialLaserFrames,
        specialLaserDrain, this, data->spriteSpecial));
      crew--;
    }
    return(TRUE);
  } else
    return(FALSE);
}

KzerZaMissile::KzerZaMissile(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite) :
  Missile(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite) {
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 50;
}

KzerZaFighter::KzerZaFighter(double ox, double oy, double oangle, double ov,
  double orange, int oarmour, int oair_frames,
  int olaser_damage, int olaser_color, double olaser_range, int olaser_frames,
  int odrain, Ship *oship, SpaceSprite *osprite) :
  Missile(ox, oy, oangle, ov, 1, orange, oarmour, oship, oship, osprite),
  air_frames(oair_frames),
	max_air_frames(oair_frames),
  laser_damage(olaser_damage),
  laser_color(olaser_color),
  laser_range(olaser_range),
  laser_frames(olaser_frames),
  batt(0),
  recharge_frames(odrain)
{
}

void KzerZaFighter::calculate() {
	Missile::calculate();

	if (!ship) {
		state = 0;
		return;
		}
	air_frames -= frame_time;
	if (air_frames <= 0) state = 0;
	if (air_frames > max_air_frames - 350) return;

	if((ship->target == NULL) || (air_frames < (max_air_frames >> 1)) || ship->target->isAlly(ship)) {
		collide_flag_ally = 1 << LAYER_SHIPS;
		angle = trajectory_angle(ship);
		vx = v * cos(angle * ANGLE_RATIO);
		vy = v * sin(angle * ANGLE_RATIO);
		sprite_index = (int)(angle / 5.625) + 16;
		sprite_index &= 63;
		return;
		}

  if ((distance(ship->target) < laser_range) && !ship->target->isAlly(ship) && (batt <= 0)) {
    collide_flag_ally = 0;
    vx = 0.0;
    vy = 0.0;
      stop_sample(ship->data->sampleExtra[0]);
      tw_sample(ship->data->sampleExtra[0]);
      spaceFrame->addSpaceLine(new Laser(trajectory_angle(ship->target),
        pallete_color[laser_color], laser_range, laser_damage,
        laser_frames, this, 0.0, -(height() / 2.0)));
      batt = recharge_frames;
    }
  else {
    SpaceLocation *L, *R;

    L = new SpaceLocation(
      ship->target->normal_x() + cos((90+ship->target->get_angle()) * ANGLE_RATIO) * laser_range * 0.8,
      ship->target->normal_y() + sin((90+ship->target->get_angle()) * ANGLE_RATIO) * laser_range * 0.8,
      0, 0);
    R = new SpaceLocation(
      ship->target->normal_x() + cos((270+ship->target->get_angle()) * ANGLE_RATIO) * laser_range * 0.8,
      ship->target->normal_y() + sin((270+ship->target->get_angle()) * ANGLE_RATIO) * laser_range * 0.8,
      0, 0);

    if (distance(L) < distance(R)) angle = trajectory_angle(L);
    else angle = trajectory_angle(R);
    delete L, R;
    vx = v * cos(angle * ANGLE_RATIO);
    vy = v * sin(angle * ANGLE_RATIO);
    sprite_index = (int)(angle / 5.625) + 16;
  	sprite_index &= 63;
    }
  if (batt) batt -= frame_time;
  
}


void KzerZaFighter::handle_damage(SpaceLocation *source){
	if (source->isPlanet()) {
		state = 1;
		return;
		}
	state = 0;
  damage = 0;
  directDamage = 0;
}

void KzerZaFighter::inflict_damage(SpaceObject *other) {
	if(other == ship) {
		stop_sample(ship->data->sampleExtra[1]);
		tw_sample(ship->data->sampleExtra[1]);
		ship->repair++;
		ship->handle_damage(this);
		} 
	if (!other->isPlanet()) state = 0;
	else {
		double a = trajectory_angle(other) * PI / 180;
		translate(cos(a) * -10, sin(a) * -10);
		}
	return;
	}

Ship *getKzerZaDreadnought(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new KzerZaDreadnought(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO KzerZaDreadnoughtInfo =
{
  "kzedr",
  "Kzer-Za Dreadnought",
  getKzerZaDreadnought
};
