
#include "../shipstuf.h"

class MyconPodship : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  double       weaponHome;
  int          specialRepair;

  public:
  MyconPodship(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int activate_weapon();
  virtual int activate_special();
};

class MyconPlasma : public HomingMissile {
  double v;

	static SpaceSprite *spriteWeaponExplosion;
  int frame_count;
  int max_damage;

  public:
  MyconPlasma(double ox, double oy, double oangle, double ov, int odamage,
    double orange, double otrate, Ship *oship, SpaceSprite *osprite, int ofcount);

  virtual void calculate();

  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage(SpaceLocation *source);
};


MyconPodship::MyconPodship(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponHome     = scale_turning(get_config_float("Weapon", "Homing", 0));
  specialRepair  = get_config_int("Special", "Repair", 0);
	collide_flag_ally |= bit(LAYER_SHOTS);
}

int MyconPodship::activate_weapon()
{
  spaceFrame->addSpaceObject(new MyconPlasma(0.0, height(),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponHome, this,
    data->spriteWeapon, 64));
  return(TRUE);
}

int MyconPodship::activate_special()
{
  if(crew >= crew_max)
    return(FALSE);
  repair += specialRepair;
  handle_damage(this);
  return(TRUE);
}

SpaceSprite *MyconPlasma::spriteWeaponExplosion = NULL;
MyconPlasma::MyconPlasma(double ox, double oy, double oangle, double ov,
    int odamage, double orange, double otrate, Ship *oship,
    SpaceSprite *osprite, int ofcount) :
  HomingMissile(ox, oy, oangle, ov, odamage, orange, 0, otrate, oship,
      osprite),
  v(ov),
  frame_count(ofcount),
  max_damage(odamage)
{
	spriteWeaponExplosion = ship->data->spriteWeaponExplosion;
  collide_flag_ally |= bit(LAYER_SHIPS);
}

void MyconPlasma::calculate()
{
  HomingMissile::calculate();

  sprite_index = (int)((d / range) * (double)(frame_count - 1));
  damage_factor = max_damage - (int)((d / range) * (double)(max_damage));
}

void MyconPlasma::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other);
  if(other->getID() != SPACE_SHOT) {
    spaceFrame->addSpaceObject(new FixedAnimation(other,
      spriteWeaponExplosion, 20, 50, LAYER_EXPLOSIONS));
    state = 0;
  }
}

void MyconPlasma::handle_damage(SpaceLocation *source) {
  int total_damage = 0;

  if((damage > 0) || (directDamage > 0)) {
    total_damage += damage;
    total_damage += directDamage;
    d += ((double)(total_damage) / (double)(max_damage) * range);
    damage = 0;
    directDamage = 0;
    if(d >= range)
      state = 0;
  }
}

Ship *getMyconPodship(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new MyconPodship(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO MyconPodshipInfo =
{
  "mycpo",
  "Mycon Podship",
  getMyconPodship
};
