
#include "../shipstuf.h"

#include "../melee/mmain.h"

#define MAX_MARINES 8

class OrzNemesis;
class OrzMissile;
class OrzMarine;

class OrzNemesis : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialAccelRate;
  double       specialSpeedMax;
  int          specialHotspotRate;
  int          specialArmour;
  OrzMarine   *marine[MAX_MARINES];

  double       turretAngle;

  public:
  OrzNemesis(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void calculate();
  virtual void animate(Frame *space);

  virtual void calculate_turn_left();
  virtual void calculate_turn_right();

  virtual int activate_weapon();
  virtual int activate_special();
};

class OrzMissile : public Missile {
  public:
  OrzMissile(double oangle, double ov, int odamage, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite);
};

class OrzMarine : public SpaceObject {
  double accel_rate;
  double speed_max;
  int    hotspot_rate;
  int    hotspot_frame;
  int    armour;
	Ship  *invading;
  int    returning;
  int    slot;
  int    damage_frame;
  Ship  *ship;

  public:
  OrzMarine(double ox, double oy, Ship *oship, double oAccelRate,
    double oSpeedMax, int oArmour, int oHotspotRate, int oSlot,
    SpaceSprite *osprite);

  void calculate();
  void animate(Frame *space);
  void animate_radar(Frame *radar);

  void inflict_damage(SpaceObject *other);
  void handle_damage(SpaceLocation *source);
};

OrzNemesis::OrzNemesis(double ox, double oy, double shipAngle,
  int team, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, team, shipRadarColor, shipData, cntrl)
{
  int i;

	collide_flag_ally = bit(LAYER_SPECIAL);
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialSpeedMax    = scale_velocity(get_config_float("Special", "SpeedMax", 0));
  double raw_specialHotspotRate = get_config_int("Special", "HotspotRate", 0);
	specialHotspotRate = scale_frames(raw_specialHotspotRate);
  specialAccelRate   = scale_acceleration(get_config_float("Special", "AccelRate", 0), raw_specialHotspotRate);
  specialArmour      = get_config_int("Special", "Armour", 0);

  for(i = 0; i < MAX_MARINES; i++)
    marine[i] = NULL;

  turretAngle = 0.0;
}

void OrzNemesis::calculate_turn_left()
{
  if(!fire_special)
    Ship::calculate_turn_left();
}

void OrzNemesis::calculate_turn_right()
{
  if(!fire_special)
    Ship::calculate_turn_right();
}

int OrzNemesis::activate_weapon()
{
  if(fire_special)
    return(FALSE);
  spaceFrame->addSpaceObject(new OrzMissile(
    floor((angle + turretAngle) / 5.625) * 5.625, weaponVelocity,
    weaponDamage, weaponRange, weaponArmour, this, data->spriteWeapon));
  return(TRUE);
}

int OrzNemesis::activate_special()
{
  if(turn_left)
    turretAngle -= 1.0;
  if(turn_right)
    turretAngle += 1.0;

  if((fire_weapon) && (crew > 1)) {
    int i = 0;

    while((marine[i] != NULL) && (i < MAX_MARINES))
      i++;
    if(i < MAX_MARINES) {
      marine[i] = new OrzMarine(x, y, this, specialAccelRate,
        specialSpeedMax, specialHotspotRate, specialArmour, i, data->spriteSpecial);
      spaceFrame->addSpaceObject(marine[i]);
      crew--;
      return(TRUE);
    }
  }

  return(FALSE);
}

void OrzNemesis::calculate()
{
  int i;

  for(i = 0; i < MAX_MARINES; i++)
    if((marine[i]) && (!marine[i]->exists()))
      marine[i] = NULL;

  Ship::calculate();
}

void OrzNemesis::animate(Frame *space)
{
  int turret_index = (int)((angle + turretAngle) / 5.625) + 16;
	turret_index &= 63;

  Ship::animate(space);
  if(phase_count == PHASE_MAX)
    data->spriteExtra->animate((int)(corner_x()), (int)(corner_y()),
      turret_index, space_zoom, space);
}

OrzMissile::OrzMissile(double oangle, double ov, int odamage, double orange,
  int oarmour, Ship *oship, SpaceSprite *osprite) :
  Missile(0.0, 0.0, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite)
{
  x += cos(angle * ANGLE_RATIO) * 30.0;
  y += sin(angle * ANGLE_RATIO) * 30.0;

  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 40;

  spaceFrame->addSpaceObject(new Animation(x, y,
    ship->data->spriteExtraExplosion, 20, 2, LAYER_EXPLOSIONS));
}

OrzMarine::OrzMarine(double ox, double oy, Ship *oship, double oAccelRate,
  double oSpeedMax, int oHotspotRate, int oArmour, int oSlot,
  SpaceSprite *osprite) :
  SpaceObject(ox, oy, 0.0, oship->allyFlag(), osprite),
  accel_rate(oAccelRate),
  speed_max(oSpeedMax),
  hotspot_rate(oHotspotRate),
  hotspot_frame(0),
  armour(oArmour),
  invading(NULL),
  returning(FALSE),
  slot(oSlot),
  damage_frame(-1),
  ship(oship)
{
		layer = LAYER_SPECIAL;
}

void OrzMarine::calculate() {
	int    chance;

	if(!ship->exists()) {
		if(invading) {
			blit(invading->data->bitmapPanel[0], invading->panelBitmap[0],
					14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6),
					14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6), 12, 12);
			invading->update_panel = TRUE;
			}
		ship = NULL;
		state = 0;
		return;
		}

	if(invading) {
		if (invading->exists()) {
			x = invading->normal_x();
			y = invading->normal_y();

			if(damage_frame > 0) {
				damage_frame -= frame_time;
				if(damage_frame <= 0) {
					draw_sprite(invading->panelBitmap[0],	sprite->get_bitmap(0), 
							14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6));
					invading->update_panel = TRUE;
					}
				return;
				}

			chance = random() % 10000;
			if (chance < 9 * frame_time) {
				invading->directDamage++;
				tw_sample(ship->data->sampleExtra[0]);
				invading->handle_damage(this);
				damage_frame = 50;
        draw_sprite(invading->panelBitmap[0], sprite->get_bitmap(1), 
						14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6));
				invading->update_panel = TRUE;
				}
			else if (chance < 10 * frame_time) {
				state = 0;
				tw_sample(ship->data->sampleExtra[2], 255, 128, 1000);
				blit(invading->data->bitmapPanel[0], invading->panelBitmap[0],
						14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6),
						14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6), 12, 12);
				invading->update_panel = TRUE;
				return;
				}
			} 
		else {
			invading = NULL;
			returning = TRUE;
			sprite_index = 1;
			}
		return;
		}

	if((ship) && (returning)) {
		angle = trajectory_angle(ship);
		collide_flag_ally = bit(LAYER_SHIPS);
		collide_flag_anyone = 0xff;
		}
	else {
		if((ship) && (ship->target) && (!ship->target->isAlly(ship)))
			angle = trajectory_angle(ship->target);
		}

/*	alpha = atan2(vy, vx);
	if((vx + driftvx) < 0.0)
		alpha += PI;
	alpha = ((alpha + PI) / (PI * 2.0)) * 360.0;
	alpha = normalize(alpha, 360.0);

	v = sqrt((vx * vx) + (vy * vy));
	if((fabs(min_delta(alpha, angle, 360.0)) > 0.5) || (v < speed_max)) {*/
	if (true) {
		accelerate (this, angle, accel_rate * frame_time, speed_max);
		if(hotspot_frame <= 0) {
			spaceFrame->addSpaceObject(new Animation(
				normal_x() - (cos(angle * ANGLE_RATIO) * w / 2.5),
				normal_y() - (sin(angle * ANGLE_RATIO) * h / 2.5), hotspotSprite,
				HOTSPOT_FRAMES, 50, LAYER_HOTSPOTS));
			hotspot_frame += hotspot_rate;
			}
		}
	if(hotspot_frame > 0) hotspot_frame-= frame_time;
	SpaceObject::calculate();
	}

void OrzMarine::animate(Frame *space)
{
  if(!invading) SpaceObject::animate(space);
}

void OrzMarine::animate_radar(Frame *radar)
{
  if(invading) SpaceObject::animate_radar(radar);
}

void OrzMarine::inflict_damage(SpaceObject *other) {
	if(((other->getID() & SPACE_MASK) == SPACE_SHIP) && (!returning)) {
		invading = (Ship *) other;
    collide_flag_anyone = 0;
    tw_sample(ship->data->sampleExtra[1], 255, 128, 1000);
    draw_sprite(invading->panelBitmap[0], sprite->get_bitmap(0), 
				14 + ((slot % 4) * 6), 16 + ((slot / 4) * 6));
		invading->update_panel = TRUE;
		}
	if((ship) && (other == ship) && (returning)) {
		state = 0;
		ship->repair++;
		ship->handle_damage(this);
		}
	}

void OrzMarine::handle_damage(SpaceLocation *source)
{
  if((damage > 0) || (directDamage > 0)) {
    armour -= damage;
    armour -= directDamage;
    damage = 0;
    directDamage = 0;
    if(armour <= 0)
      state = 0;
  }
}

Ship *getOrzNemesis(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new OrzNemesis(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO OrzNemesisInfo =
{
  "orzne",
  "Orz Nemesis",
  getOrzNemesis
};
