
#include "../shipstuf.h"

class PkunkFury : public Ship {
  int reborn;

  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  PkunkFury(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual void handle_damage(SpaceLocation *source);
  virtual void calculate_phase();
  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

PkunkFury::PkunkFury(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

	reborn = 0;
}

void PkunkFury::handle_damage(SpaceLocation *source) {
  Ship::handle_damage(source);

  if (exists()) return;

  if ((random() % 2) == 0) {
    x = (double)(random() % (int)(3001)) - 1500;
    y = (double)(random() % (int)(3001)) - 1500;
    if(distance(spacePlanet) < 1000.0) {
      x += cos(trajectory_angle(spacePlanet) * ANGLE_RATIO) * 1000.0;
      y += sin(trajectory_angle(spacePlanet) * ANGLE_RATIO) * 1000.0;
    }

    meleeVictory = NULL;
    state = 1;
		vx = vy = 0.0;
    crew = crew_max;
    batt = batt_max;
    phase_count = 0;
    reborn = TRUE;
    update_panel = TRUE;
    tw_sample(data->sampleExtra[0]);
  }
}

void PkunkFury::calculate_phase()
{
  if(phase_count < PHASE_MAX) {
    if(phase_step > 0)
      phase_step -= frame_time;
    else {
      phase_step += PHASE_DELAY;
      spaceFrame->addSpaceObject(new Animation(
        x - (cos(angle * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
        y - (sin(angle * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
        phaseSprite, HOT_COLORS, 50, LAYER_SHIPS));
      if(reborn) {
        spaceFrame->addSpaceObject(new Animation(
          x - (cos((angle + 90.0) * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
          y - (sin((angle + 90.0) * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
          phaseSprite, HOT_COLORS, 50, LAYER_SHIPS));
        spaceFrame->addSpaceObject(new Animation(
          x - (cos((angle + 180.0) * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
          y - (sin((angle + 180.0) * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
          phaseSprite, HOT_COLORS, 50, LAYER_SHIPS));
        spaceFrame->addSpaceObject(new Animation(
          x - (cos((angle + 270.0) * ANGLE_RATIO) * w * (double)(PHASE_MAX - phase_count)),
          y - (sin((angle + 270.0) * ANGLE_RATIO) * h * (double)(PHASE_MAX - phase_count)),
          phaseSprite, HOT_COLORS, 50, LAYER_SHIPS));
				reborn = 0;
      }
      phase_count++;
    }
  }
}

int PkunkFury::activate_weapon()
{
  spaceFrame->addSpaceObject(new AnimatedShot(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, this, data->spriteWeapon, 10, 1, 1.0));
  spaceFrame->addSpaceObject(new AnimatedShot(
    -(width() / 2.0), 0.0, angle - 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1, 1.0));
  spaceFrame->addSpaceObject(new AnimatedShot(
    width() / 2.0, 0.0, angle + 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, this, data->spriteWeapon, 10, 1, 1.0));

  return(TRUE);
}

void PkunkFury::calculate_fire_special()
{
  int i;

  if(fire_special) {
    if((special_recharge != 0) || (batt >= batt_max))
      return;

    batt += special_drain;
    if(batt > batt_max)
      batt = batt_max;

    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[special_sample]);

    do {
      i = random() % 6;
    } while(i == special_sample);
    special_sample = i;

    tw_sample(data->sampleSpecial[special_sample]);
  }
}

Ship *getPkunkFury(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new PkunkFury(sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO PkunkFuryInfo =
{
  "pkufu",
  "Pkunk Fury",
  getPkunkFury
};
