
#include "../shipstuf.h"

#include "../frame.h"

class UtwigJugger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  UtwigJugger(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual void calculate_fire_weapon();
  virtual void animate(Frame *space);
  virtual void handle_damage(SpaceLocation *source);
};

UtwigJugger::UtwigJugger(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
}

void UtwigJugger::calculate_fire_weapon()
{
  weapon_low = FALSE;

  if(fire_weapon) {
    if((weapon_recharge > 0) || (special_recharge > 0))
      return;
    if (fire_special && batt) return;
    
    spaceFrame->addSpaceObject(new Missile(34.0, 11.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(new Missile(-34.0, 11.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(new Missile(18.0, 20.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(new Missile(-18.0, 20.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(new Missile(6.0, 27.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(new Missile(-6.0, 27.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));

    weapon_recharge = weapon_rate;

    stop_sample(data->sampleWeapon[0]);
    tw_sample(data->sampleWeapon[0]);
  }
}

void UtwigJugger::animate(Frame *space)
{
  if (phase_count < PHASE_MAX)
    return;
  if (special_recharge > 0)
    sprite->animate_character((int)(corner_x()), (int)(corner_y()), sprite_index, pallete_color[hot_color[((int)(special_recharge/10)) % HOT_COLORS]], space_zoom, space);
  else
    sprite->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space_zoom, space);
}

void UtwigJugger::handle_damage(SpaceLocation *source)
{
  if(special_recharge > 0) {
    batt += damage;
    if(batt > batt_max)
      batt = batt_max;
    damage = 0;
  }
  Ship::handle_damage(source);
}

Ship *getUtwigJugger(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new UtwigJugger(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO UtwigJuggerInfo =
{
  "utwju",
  "Utwig Jugger",
  getUtwigJugger
};
