
#include "../shipstuf.h"

class VuxIntruder : public Ship {
  int    weaponColor;
  double weaponRange;
  int    weaponDamage;

  double specialRange;
  double specialVelocity;
  double specialSlowdown;
  int    specialArmour;
  
  int    engage;

  public:
  VuxIntruder(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData, Control *cntrl);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void materialize();
  virtual void calculate_phase();
};

class VuxLimpet : public AnimatedShot {
  double slowdown_factor;

  public:
  VuxLimpet(double ox, double oy, double ov, double slowdown, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

VuxIntruder::VuxIntruder(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl),
  engage(TRUE)

{
  weaponColor  = get_config_int("Weapon", "Color", 0);
  weaponRange  = scale_range(get_config_float("Weapon", "Range", 0));
  weaponDamage = get_config_int("Weapon", "Damage", 0);

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialSlowdown = get_config_float("Special", "Slowdown", 1);
  specialArmour   = get_config_int("Special", "Armour", 0);
}

int VuxIntruder::activate_weapon()
{
  spaceFrame->addSpaceLine(new Laser(angle,
    pallete_color[weaponColor], weaponRange, weaponDamage, weapon_rate,
    this, 0.0, (h / 2.0)));
  return(TRUE);
}

int VuxIntruder::activate_special()
{
  spaceFrame->addSpaceObject(new VuxLimpet(0.0, -height() / 2.0,
    specialVelocity, specialSlowdown, specialRange, specialArmour, this,
    data->spriteSpecial, 100, 5));
  return(TRUE);
}

VuxLimpet::VuxLimpet(double ox, double oy, double ov, double slowdown,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofsize, int ofcount) :
  AnimatedShot(ox, oy, 0.0, ov, 0, orange, oarmour, oship, oship, osprite,
    ofcount, ofsize),
  slowdown_factor(slowdown)
{
  if((ship->target) && (!ship->target->isAlly(ship)))
    angle = trajectory_angle(ship->target);
  else
    angle = ship->get_angle() - 180.0;

  vx    = v * cos(angle * ANGLE_RATIO);
  vy    = v * sin(angle * ANGLE_RATIO);

  if(!ship->exists())
    state = 0;
}

void VuxLimpet::calculate()
{
//	if(!ship->exists()) {
//		state = 0;
//		return;
//		}

	if(ship && (ship->target) && (!ship->target->isAlly(ship))) {
		angle = trajectory_angle(ship->target);
		vx = v * cos(angle * ANGLE_RATIO);
		vy = v * sin(angle * ANGLE_RATIO);
		}
	AnimatedShot::calculate();
	}

void VuxLimpet::inflict_damage(SpaceObject *other) {
	if(!other->isShip()) {
		if (other->damage_factor || other->mass) state = 0;
		return;
		}

	Ship *target = (Ship *) other;

	tw_sample(ship->data->sampleSpecial[1]);

	//MYCODE begin
	BITMAP *bmp=target->getSprite()->get_bitmap(0);
	double ang,dist;
	int hx,hy,tries;

	//find a random spot on the target ship where it "exist" 
	tries = 0;
	do {
		ang=double(random() % 360);
		dist=double(random() % 100)/100.0;
		hx=int((0.50+cos(ang*ANGLE_RATIO)*0.35*dist)*double(bmp->w) );
		hy=int((0.50+sin(ang*ANGLE_RATIO)*0.35*dist)*double(bmp->h) );
		tries++; // just for robustness... don't try forever.
		} 
	while ( (getpixel(bmp,hx,hy)==bitmap_mask_color(bmp)) && (tries<32) );

	// draw the Limplet on the ship panel
	BITMAP *s = sprite->get_bitmap(sprite_index);
	stretch_sprite(
		target->panelBitmap[0],
		s,
		hx - 6 - s->w/4, 
		hy - s->h/4,
		s->w/2, s->h/2 );
	target->update_panel = TRUE;

  // draw the Limplet on the ship!
/*	for (int i=0; i<64; i++) {
		draw_sprite(target->getSprite()->get_bitmap(i),
			sprite->get_bitmap(sprite_index),
			int((0.50+cos((i*360.0/64.0+ang)*ANGLE_RATIO)*0.35*dist)*double(bmp->w) )-4,  
			int((0.50+sin((i*360.0/64.0+ang)*ANGLE_RATIO)*0.35*dist)*double(bmp->h) )-4
		);
	}*/

	target->speed_loss += slowdown_factor;
	target->handle_damage(this);
	state = 0;
	}

void VuxIntruder::materialize()
{
  BITMAP *bmp[HOT_COLORS];
  int    i;

  if(engage)
   {
    if (target) 
     {
       x = target->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
       y = target->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);
       angle = trajectory_angle(target);
     }

    sprite_index = (int)(angle / 5.625) + 16;
  	sprite_index &= 63;

    phase_count = 0;
    phase_step  = 0;

    for(i = 0; i < HOT_COLORS; i++) {
      bmp[i] = create_bitmap((int)(w), (int)(h));
      clear_to_color(bmp[i], bitmap_mask_color(bmp[i]));
      sprite->draw_character(0, 0, sprite_index, pallete_color[hot_color[i]], bmp[i]);
    };

    delete phaseSprite;
    phaseSprite = new SpaceSprite(bmp, HOT_COLORS);

    for(i = 0; i < HOT_COLORS; i++)
      destroy_bitmap(bmp[i]);

    engage = FALSE;
   }
}

void VuxIntruder::calculate_phase()
{
  if (engage)
   materialize(); 
  else
   Ship::calculate_phase();
}

Ship *getVuxIntruder(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl) {
  return(new VuxIntruder(sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO VuxIntruderInfo =
{
  "vuxin",
  "Vux Intruder",
  getVuxIntruder
};
