
#include "../shipstuf.h"

#include "../frame.h"

class YehatTerminator : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int          specialFrames;
  int          shieldFrames;

  public:
  YehatTerminator(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void animate(Frame *space);
  virtual void handle_damage(SpaceLocation *source);
};

YehatTerminator::YehatTerminator(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData, Control *cntrl) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData, cntrl)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialFrames = get_config_int("Special", "Frames", 0);
  shieldFrames  = 0;
}

int YehatTerminator::activate_weapon()
{
  spaceFrame->addSpaceObject(new Missile(-24.0, 14,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  spaceFrame->addSpaceObject(new Missile(24.0, 14,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  return(TRUE);
}

int YehatTerminator::activate_special()
{
  shieldFrames = (shieldFrames % frame_time) + specialFrames;
  return(TRUE);
}

void YehatTerminator::calculate()
{
   if(shieldFrames > 0)
     shieldFrames-= frame_time;
   Ship::calculate();
}

void YehatTerminator::animate(Frame *space)
{
  if(phase_count < PHASE_MAX)
    return;
  if(shieldFrames > 0)
    data->spriteSpecial->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space_zoom, space);
  else
    sprite->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space_zoom, space);
}

void YehatTerminator::handle_damage(SpaceLocation *source)
{
  if(shieldFrames > 0) damage = 0;
  Ship::handle_damage(source);
}

Ship *getYehatTerminator(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData, Control *cntrl)
{
  return(new YehatTerminator(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData, cntrl));
}

SHIP_REGISTER_INFO YehatTerminatorInfo =
{
  "yehte",
  "Yehat Terminator",
  getYehatTerminator
};
